@@ -15,11 +15,15 @@ import mimetypes
 import grpc
 from argparse import ArgumentParser, Namespace
 from typing import Dict, Generator, List, Optional, Union, Any, Sequence, Tuple
-from dotenv import load_dotenv
 from google.protobuf.json_format import MessageToJson
 from PIL import Image
 
-load_dotenv()
+try:
+    from dotenv import load_dotenv
+except ModuleNotFoundError:
+    pass
+else:
+    load_dotenv()
 
 thisPath = pathlib.Path(__file__).parent.resolve()
 genPath = thisPath / "interfaces/gooseai/generation"
@@ -28,6 +32,8 @@ sys.path.append(str(genPath))
 import stability_sdk.interfaces.gooseai.generation.generation_pb2 as generation
 import stability_sdk.interfaces.gooseai.generation.generation_pb2_grpc as generation_grpc
 
+MAX_FILENAME_SZ = int(os.getenv("MAX_FILENAME_SZ", 200))
+
 logger = logging.getLogger(__name__)
 logger.setLevel(level=logging.INFO)
 
@@ -42,6 +48,13 @@ algorithms: Dict[str, int] = {
     "k_lms": generation.SAMPLER_K_LMS,
 }
 
+def truncate_fit(prefix: str, prompt: str, ext: str, ts: int, idx: int, max: int) -> str:
+    post = f"_{ts}_{idx}"
+    prompt_budget = max
+    prompt_budget -= len(prefix)
+    prompt_budget -= len(post)
+    prompt_budget -= len(ext) + 1
+    return f"{prefix}{prompt[:prompt_budget]}{post}{ext}"
 
 def image_to_prompt(im, init: bool = False, mask: bool = False) -> generation.Prompt:
     if init and mask:
@@ -79,6 +92,7 @@ def get_sampler_from_str(s: str) -> generation.DiffusionSampler:
 
 def process_artifacts_from_answers(
     prefix: str,
+    prompt: str,
     answers: Union[
         Generator[generation.Answer, None, None], Sequence[generation.Answer]
     ],
@@ -89,6 +103,7 @@ def process_artifacts_from_answers(
     Process the Artifacts from the Answers.
 
     :param prefix: The prefix for the artifact filenames.
+    :param prompt: The prompt to add to the artifact filenames.
     :param answers: The Answers to process.
     :param write: Whether to write the artifacts to disk.
     :param verbose: Whether to print the artifact filenames.
@@ -98,7 +113,7 @@ def process_artifacts_from_answers(
     idx = 0
     for resp in answers:
         for artifact in resp.artifacts:
-            artifact_p = f"{prefix}-{resp.request_id}-{resp.answer_id}-{idx}"
+            artifact_start = time.time()
             if artifact.type == generation.ARTIFACT_IMAGE:
                 ext = mimetypes.guess_extension(artifact.mime)
                 contents = artifact.binary
@@ -111,7 +126,7 @@ def process_artifacts_from_answers(
             else:
                 ext = ".pb"
                 contents = artifact.SerializeToString()
-            out_p = f"{artifact_p}{ext}"
+            out_p = truncate_fit(prefix, prompt, ext, int(artifact_start), idx, MAX_FILENAME_SZ)
             if write:
                 with open(out_p, "wb") as f:
                     f.write(bytes(contents))
@@ -199,7 +214,7 @@ class StabilityInference:
 
     def generate(
         self,
-        prompt: Union[List[str], str],
+        prompt: Union[str, List[str], generation.Prompt, List[generation.Prompt]],
         init_image: Optional[Image.Image] = None,
         mask_image: Optional[Image.Image] = None,
         height: int = 512,
@@ -213,6 +228,11 @@ class StabilityInference:
         samples: int = 1,
         safety: bool = True,
         classifiers: Optional[generation.ClassifierParameters] = None,
+        guidance_preset: generation.GuidancePreset = generation.GUIDANCE_PRESET_NONE,
+        guidance_cuts: int = 0,
+        guidance_strength: Optional[float] = None,
+        guidance_prompt: Union[str, generation.Prompt] = None,
+        guidance_models: List[str] = None,
     ) -> Generator[generation.Answer, None, None]:
         """
         Generate images from a prompt.
@@ -231,6 +251,11 @@ class StabilityInference:
         :param samples: Number of samples to generate.
         :param safety: DEPRECATED/UNUSED - Cannot be disabled.
         :param classifiers: DEPRECATED/UNUSED - Has no effect on image generation.
+        :param guidance_preset: Guidance preset to use. See generation.GuidancePreset for supported values.
+        :param guidance_cuts: Number of cuts to use for guidance.
+        :param guidance_strength: Strength of the guidance. We recommend values in range [0.0,1.0]. A good default is 0.25
+        :param guidance_prompt: Prompt to use for guidance, defaults to `prompt` argument (above) if not specified.
+        :param guidance_models: Models to use for guidance.
         :return: Generator of Answer objects.
         """
         if (prompt is None) and (init_image is None):
@@ -250,42 +275,72 @@ class StabilityInference:
         else:
             seed = list(seed)
 
-        prompt_ = []
-        if isinstance(prompt, str):
-            prompt_ = [generation.Prompt(text=prompt)]
-        elif isinstance(prompt, Sequence):
-            prompt_ = [generation.Prompt(text=p) for p in prompt]
-        else:
-            raise TypeError("prompt must be a string or a sequence")
+        prompts: List[generation.Prompt] = []
+        if any(isinstance(prompt, t) for t in (str, generation.Prompt)):
+            prompt = [prompt]
+        for p in prompt:
+            if isinstance(p, str):
+                p = generation.Prompt(text=p)
+            elif not isinstance(p, generation.Prompt):
+                raise TypeError("prompt must be a string or generation.Prompt object")
+            prompts.append(p)
 
+        step_parameters = dict(
+            scaled_step=0,
+            sampler=generation.SamplerParameters(cfg_scale=cfg_scale),
+        )
+            
         if init_image is not None:
-            prompt_ += [image_to_prompt(init_image, init=True)]
-            parameters = (
-                generation.StepParameter(
-                    scaled_step=0,
-                    sampler=generation.SamplerParameters(
-                        cfg_scale=cfg_scale,
-                    ),
-                    schedule=generation.ScheduleParameters(
-                        start=start_schedule,
-                        end=end_schedule,
-                    ),
-                ),
-            )
-            if mask_image is not None:
-                prompt_ += [image_to_prompt(mask_image, mask=True)]
-        else:
-            parameters = (
-                generation.StepParameter(
-                    scaled_step=0,
-                    sampler=generation.SamplerParameters(cfg_scale=cfg_scale),
-                ),
+            step_parameters['schedule'] = generation.ScheduleParameters(
+                start=start_schedule,
+                end=end_schedule,
             )
+            prompts += [image_to_prompt(init_image, init=True)]
 
+            if mask_image is not None:
+                prompts += [image_to_prompt(mask_image, mask=True)]
+
+        
+        if guidance_prompt:
+            if isinstance(guidance_prompt, str):
+                guidance_prompt = generation.Prompt(text=guidance_prompt)
+            elif not isinstance(guidance_prompt, generation.Prompt):
+                raise ValueError("guidance_prompt must be a string or Prompt object")
+        if guidance_strength == 0.0:
+            guidance_strength = None
+
+            
+        # Build our CLIP parameters
+        if guidance_preset is not generation.GUIDANCE_PRESET_NONE:
+            # to do: make it so user can override this
+            step_parameters['sampler']=None
+
+            if guidance_models:
+                guiders = [generation.Model(alias=model) for model in guidance_models]
+            else:
+                guiders = None
+
+            if guidance_cuts:
+                cutouts = generation.CutoutParameters(count=guidance_cuts)
+            else:
+                cutouts = None
+
+            step_parameters["guidance"] = generation.GuidanceParameters(
+                guidance_preset=guidance_preset,
+                instances=[
+                    generation.GuidanceInstanceParameters(
+                        guidance_strength=guidance_strength,
+                        models=guiders,
+                        cutouts=cutouts,
+                        prompt=guidance_prompt,
+                    )
+                ],
+            )
+            
         rq = generation.Request(
             engine_id=self.engine,
             request_id=request_id,
-            prompt=prompt_,
+            prompt=prompts,
             image=generation.ImageParameters(
                 transform=generation.TransformType(diffusion=sampler),
                 height=height,
@@ -293,7 +348,7 @@ class StabilityInference:
                 seed=seed,
                 steps=steps,
                 samples=samples,
-                parameters=parameters,
+                parameters=[generation.StepParameter(**step_parameters)],
             ),
         )
 
@@ -403,7 +458,7 @@ if __name__ == "__main__":
         "--prefix",
         "-p",
         type=str,
-        default="generation",
+        default="generation_",
         help="output prefixes for artifacts",
     )
     parser.add_argument(
@@ -456,7 +511,7 @@ if __name__ == "__main__":
 
     answers = stability_api.generate(args.prompt, **request)
     artifacts = process_artifacts_from_answers(
-        args.prefix, answers, write=not args.no_store, verbose=True
+        args.prefix, args.prompt, answers, write=not args.no_store, verbose=True
     )
     if args.show:
         for artifact in open_images(artifacts, verbose=True):
